
class ApiError(Exception):
    """Error class for web API related Exceptions"""
    pass


class ConfigError(Exception):
    """Error class for config related Exceptions"""
    pass


class CondaError(Exception):
    """Error class for Anaconda-related exceptions"""
    pass
